#include "conf.h"
#include "modcmd.h"
#include "saxdb.h"
#include "timeq.h"

int bad;
const char *hidden_host_suffix;

/* because recdb likes to log stuff.. */
void log_module(struct log_type *lt, enum log_severity ls, const char *format, ...)
{
    va_list va;
    (void)lt; (void)ls;
    va_start(va, format);
    vfprintf(stderr, format, va);
    va_end(va);
    bad = 1;
}

/* and because saxdb is tied in to lots of stuff.. */

time_t now;

void *conf_get_data(const char *full_path, enum recdb_type type) {
    (void)full_path; (void)type; return NULL;
}

void conf_register_reload(conf_reload_func crf) {
    (void)crf;
}

void reg_exit_func(exit_func_t handler) {
    (void)handler;
}

void timeq_add(time_t when, timeq_func func, void *data) {
    (void)when; (void)func; (void)data;
}

void timeq_del(time_t when, timeq_func func, void *data, int mask) {
    (void)when; (void)func; (void)data; (void)mask;
}


int send_message(struct userNode *dest, struct userNode *src, const char *message, ...) {
    (void)dest; (void)src; (void)message; return 0;
}

struct module *module_register(const char *name, enum log_type clog, const char *helpfile_name, expand_func_t expand_help) {
    (void)name; (void)clog; (void)helpfile_name; (void)expand_help; return NULL;
}

struct modcmd *modcmd_register(struct module *module, const char *name, modcmd_func_t func, unsigned int min_argc, unsigned int flags, ...) {
    (void)module; (void)name; (void)func; (void)min_argc; (void)flags; return NULL;
}

void table_send(struct userNode *from, const char *to, unsigned int size, irc_send_func irc_send, struct helpfile_table table) {
    (void)from; (void)to; (void)size; (void)irc_send; (void)table;
}

/* back to our regularly scheduled code: */

int check_record(const char *key, void *data, void *extra)
{
    struct record_data *rd = data;
    (void)extra;
    switch (rd->type) {
    case RECDB_INVALID:
	fprintf(stdout, "Invalid database record type for key %s\n", key);
	return 1;
    case RECDB_QSTRING:
    case RECDB_STRING_LIST:
	return 0;
    case RECDB_OBJECT:
	return dict_foreach(rd->d.object, check_record, NULL) ? 1 : 0;
    }
    return 0;
}

int main(int argc, char *argv[])
{
    dict_t db;
    char *infile;

    if (argc < 2 || argc > 3) {
        fprintf(stderr, "%s usage: %s <dbfile> [outputfile]\n\n", argv[0], argv[0]);
        fprintf(stderr, "If [outputfile] is specified, dbfile is rewritten into outputfile after being\nparsed.\n\n");
        fprintf(stderr, "<dbfile> and/or [outputfile] may be given as '-' to use stdin and stdout,\nrespectively.\n");
        return 1;
    }

    tools_init();
    if (!strcmp(argv[1], "-")) {
        infile = "/dev/stdin";
    } else {
        infile = argv[1];
    }
    if (!(db = parse_database(infile))) return 2;
    fprintf(stdout, "Database read okay.\n");
    fflush(stdout);
    if (dict_foreach(db, check_record, 0)) return 3;
    if (!bad) {
        fprintf(stdout, "Database checked okay.\n");
        fflush(stdout);
    }

    if (argc == 3) {
        FILE *f;

        if (!strcmp(argv[2], "-")) {
            f = stdout;
        } else {
            if (!(f = fopen(argv[2], "w+"))) {
                fprintf(stderr, "fopen: %s\n", strerror(errno));
                return 4;
            }
        }

        write_database(f, db);
        fclose(f);
        fprintf(stdout, "Database written okay.\n");
        fflush(stdout);
    }

    return 0;
}
